/* oauth.js
*
* This module computes OAuth1.0a signatures using HMAC-SHA1 or HMAC-SHA256.
*
* usage example:
*
* var options = {
*      version : properties.version || '1.0A',
*      method : properties.method,
*      url : properties.url,
*      consumerkey : properties.consumerkey,
*      consumersecret :properties.consumersecret,
*      paramString : properties.params,
*      nonce : properties.nonce,
*      timestamp : properties.timestamp
*    },
*    oauth = new OAuth(options),
*    normalized = oauth.computeNormalizedParameters(),
*    baseString = oauth.computeBaseString(normalized),
*    signature = oauth.computeSignature(baseString),
*    header = oauth.produceAuthzHeader(signature);
*
******/

/* jshint esversion: 6, node: false */
/* global JSSHA, sjcl */

(function (){

  function rfc3986EncodeURIComponent (str) {
    return encodeURIComponent(str).replace(/[!'()*]/g, function(c) {
      return '%' + c.charCodeAt(0).toString(16);
    });
  }

  function generateRandomString(L) {
    var c = function() {
          var m = Math.floor(Math.random() * 26),
              a = (Math.floor(Math.random() * 2) * 32);
          return String.fromCharCode(65 + m + a);
        },
        i,
        pw = '';

    L = Math.floor(Math.random() * (L || 7)) + 8;

    for (i=0; i<L; i++) {
      pw += c();
    }
    return pw;
  }

  function now() {
    var value = (new Date()).valueOf() / 1000;
    value = value.toFixed(0);
    return value;
  }


  function newNonce() {
    var length = Math.floor((Math.random() * 18 + 4));
    return generateRandomString(length);
  }

  var onEquals = new RegExp('=(?=.+)');
  function quoteValue(elt) {
    // input:  name=value
    // return: name="value"
    var pair = elt.split(onEquals, 2);
    return pair[0] + '="' + pair[1] + '"';
  }

  function isNotDefined(thing) {
    return ("[" + thing + "]" == "[undefined]");
  }

  function isDefined(thing) {
    return !(isNotDefined(thing));
  }

  var OAuth = function(options) {
        this.options = options;
        this.nonce = (isDefined(options.nonce)) ? options.nonce: newNonce();
        this.timestamp = (isDefined(options.timestamp)) ? options.timestamp : now() ;
      };

  OAuth.prototype = {
    getVersion : function() {
      return (isNotDefined(this.options.version)) ? '1.0A' : this.options.version;
    },

    getSignatureMethod : function() {
      return (isNotDefined(this.options.signature_method)) ? 'HMAC-SHA1' : this.options.signature_method;
    },

    getOauthParams : function(nonce, timestamp) {
      var self = this,
      a = [
        'consumer_key=' + this.options.consumerkey,
        'nonce=' + this.nonce,
        'timestamp=' + this.timestamp,
        'version=' + this.getVersion(),
        'signature_method=' + this.getSignatureMethod()
      ];
      ['callback', 'verifier', 'token'].forEach(function(term){
        var value = self.options[term];
        if (value) { a.push(term + '=' + rfc3986EncodeURIComponent(value)); }
      });
      return a.map(function(elt){return 'oauth_' + elt;});
    },

    getParams : function() {
      if ( ! this.paramString) return [];
      return this.paramString.split('&');
    },

    computeNormalizedParameters : function() {
      return this.getParams()
        .concat( this.getOauthParams() )
        .sort()
        .join('&');
    },

    computeBaseString : function(normalizedParams) {
      return [ this.options.method, this.options.url, normalizedParams ]
        .map(function(elt) { return rfc3986EncodeURIComponent(elt);})
        .join('&');
    },

    getKey : function() {
      var key = this.options.consumersecret + '&',
          tokenSecret = this.options.tokensecret;
      if (tokenSecret && tokenSecret!== '') { key += tokenSecret; }
      return key;
    },

    computeSignature : function(message) {
      var key = this.getKey(), signature;
      if (isDefined(sjcl)) {
        var hash = (this.getSignatureMethod() == 'HMAC-SHA1')? sjcl.hash.sha1 : sjcl.hash.sha256;
        var hmac = new sjcl.misc.hmac(sjcl.codec.utf8String.toBits(key), hash);
        signature = sjcl.codec.base64.fromBits(hmac.encrypt(message));
        return signature;
      }
      return "none";
    },

    produceAuthzHeader : function(signature) {
      // Eg, something like:
      // OAuth realm="http://sp.example.com/",
      //    oauth_consumer_key="0685bd9184jfhq22",
      //    oauth_token="ad180jjd733klru7",
      //    oauth_signature_method="HMAC-SHA1",
      //    oauth_signature="wOJIO9A2W5mFwDgiDvZbTSMK%2FPY%3D",
      //    oauth_timestamp="137131200",
      //    oauth_nonce="4572616e48616d6d65724c61686176",
      //    oauth_version="1.0"
      var oauthparams = this.getOauthParams();
      oauthparams.push('oauth_signature=' + rfc3986EncodeURIComponent(signature));
      var oauthparams_in_qparams = this.getParams().filter(function(elt){return elt.startsWith('oauth_');});
      if (oauthparams_in_qparams.length > 0) {
        oauthparams = oauthparams.concat(oauthparams_in_qparams);
      }
      oauthparams.sort();
      var realm = this.options.realm;
      var realmString = realm ? (quoteValue('realm=' + realm.trim()) + ',') : '';
      return 'OAuth ' + realmString + oauthparams.map(quoteValue).join(',');
    }

  };

  var globalScope = (function(){ return this; }).call(null);
  globalScope.OAuth = OAuth;

}());
